@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: turba/data.php,v 1.70.4.2.2.1 2005/12/10 22:08:34 chuck Exp $
+ * $Horde: turba/data.php,v 1.70.4.5 2006/01/01 21:29:15 jan Exp $
  *
- * Copyright 2001-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
@@ -15,6 +15,27 @@ function _cleanup()
     return IMPORT_FILE;
 }
 
+/**
+ * Remove empty attributes from attributes array.
+ *
+ * @param mixed $val    Value from attributes array.
+ *
+ * @return boolean         Boolean used by array_filter.
+ */
+function _emptyAttributeFilter($var)
+{
+    if (!is_array($var)) {
+        return ($var != '');
+    } else {
+        foreach ($var as $v) {
+            if ($v == '') {
+                return false;
+            }
+        }
+        return true;
+    }
+}
+
 @define('TURBA_BASE', dirname(__FILE__));
 require_once TURBA_BASE . '/lib/base.php';
 require_once 'Horde/Data.php';
@@ -82,53 +103,58 @@ if ($actionID != 'select') {
 /* Loop through the action handlers. */
 switch ($actionID) {
 case 'export':
-    $contacts = array();
+    $sources = array();
     if (Util::getFormData('selected')) {
         foreach (Util::getFormData('objectkeys') as $objectkey) {
             list($source, $key) = explode(':', $objectkey, 2);
-            if (!isset($contacts[$source])) {
-                $contacts[$source] = array();
+            if (strpos($key, ':')) {
+                list($owner, $key) = explode(':', $key, 2);
             }
-            $contacts[$source][] = $key;
+            if (!isset($sources[$source])) {
+                $sources[$source] = array();
+            }
+            $sources[$source][] = $key;
         }
     } else {
         $source = Util::getFormData('source');
-        if (!isset($source) && isset($cfgSources) &&
-            is_array($cfgSources) && count($cfgSources) > 0) {
+        if (!isset($source) && !empty($cfgSources)) {
             reset($cfgSources);
             $source = key($cfgSources);
         }
-        $contacts[$source] = array();
+        $sources[$source] = array();
     }
 
     $data = array();
+    $contacts = array();
     $all_fields = array();
-    foreach ($contacts as $source => $fields) {
+    foreach ($sources as $source => $objectkeys) {
         /* Create a Turba storage instance. */
-        $storage = &Turba_Driver::singleton($source, $cfgSources[$source]);
+        $storage = &Turba_Driver::singleton($source);
         if (is_a($storage, 'PEAR_Error')) {
             $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
             $error = true;
             break;
         }
-        /* Get the full, sorted address list. */
-        if (count($fields)) {
-            $addresses = &$storage->getObjects($fields);
+
+        /* Get the full, sorted contact list. */
+        if (count($objectkeys)) {
+            $results = &$storage->getObjects($objectkeys);
         } else {
-            $addresses = $storage->search(array());
-            if (is_a($addresses, 'Turba_List')) {
-                $addresses = $addresses->objects;
+            $results = $storage->search(array());
+            if (is_a($results, 'Turba_List')) {
+                $results = $results->objects;
             }
         }
-        if (is_a($addresses, 'PEAR_Error')) {
-            $notification->push(sprintf(_("Failed to search the directory: %s"), $addresses->getMessage()), 'horde.error');
+        if (is_a($results, 'PEAR_Error')) {
+            $notification->push(sprintf(_("Failed to search the directory: %s"), $results->getMessage()), 'horde.error');
             $error = true;
             break;
         }
+
         $fields = $storage->getFields();
         $all_fields = array_merge($all_fields, $fields);
         $params = $storage->getParams();
-        foreach ($addresses as $ob) {
+        foreach ($results as $ob) {
             $row = array();
             foreach ($fields as $field) {
                 if (substr($field, 0, 2) != '__') {
@@ -146,8 +172,10 @@ case 'export':
             }
             $data[] = $row;
         }
+
+        $contacts = array_merge($contacts, $results);
     }
-    if (count($data) == 0) {
+    if (!count($data)) {
         $notification->push(_("There were no addresses to export."), 'horde.message');
         $error = true;
         break;
@@ -155,7 +183,7 @@ case 'export':
 
     /* Make sure that all rows have the same columns if exporting from
      * different sources. */
-    if (count($contacts) > 1) {
+    if (count($sources) > 1) {
         for ($i = 0; $i < count($data); $i++) {
             foreach ($all_fields as $field) {
                 if (!isset($data[$i][$field])) {
@@ -165,8 +193,7 @@ case 'export':
         }
     }
 
-    $exportID = Util::getFormData('exportID');
-    switch ($exportID) {
+    switch (Util::getFormData('exportID')) {
     case EXPORT_CSV:
         $csv = &Horde_Data::singleton('csv');
         $csv->exportFile(_("contacts.csv"), $data, true);
@@ -181,11 +208,42 @@ case 'export':
         $tsv = &Horde_Data::singleton('tsv');
         $tsv->exportFile(_("contacts.tsv"), $data, true);
         exit;
+
+    case EXPORT_VCARD:
+        $cards = array();
+        foreach ($contacts as $contact) {
+            $cards[] =& Turba_Driver::tovCard($contact);
+        }
+
+        $vcard = &Horde_Data::singleton('vcard');
+        $vcard->exportFile(_("contacts.vcf"), $cards, true);
+        exit;
     }
     break;
 
 case IMPORT_FILE:
-    $_SESSION['import_data']['target'] = Util::getFormData('dest');
+    $dest = Util::getFormData('dest');
+    $storage = &Turba_Driver::singleton($dest);
+    if (is_a($storage, 'PEAR_Error')) {
+        $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
+        $error = true;
+        break;
+    }
+
+    /* Check permissions. */
+    $max_contacts = Turba::hasPermission($dest . ':max_contacts', 'source');
+    if ($max_contacts !== true &&
+        $max_contacts <= $storage->countContacts()) {
+        $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d contacts in \"%s\"."), $max_contacts, $storage->title), ENT_COMPAT, NLS::getCharset());
+        if (!empty($conf['hooks']['permsdenied'])) {
+            $message = Horde::callHook('_perms_hook_denied', array('turba:max_contacts'), 'horde', $message);
+        }
+        $notification->push($message, 'horde.error', array('content.raw'));
+        $error = true;
+        break;
+    }
+
+    $_SESSION['import_data']['target'] = $dest;
     $_SESSION['import_data']['purge'] = Util::getFormData('purge');
     break;
 
@@ -215,6 +273,17 @@ if (!$error && !empty($driver)) {
         if (is_a($next_step, 'PEAR_Error')) {
             $notification->push($next_step->getMessage(), 'horde.error');
             $next_step = $data->cleanup();
+        } else {
+            /* Raise warnings if some exist. */
+            if (method_exists($data, 'warnings')) {
+                $warnings = $data->warnings();
+                if (count($warnings)) {
+                    foreach ($warnings as $warning) {
+                        $notification->push($warning, 'horde.warning');
+                    }
+                    $notification->push(_("The import can be finished despite the warnings."), 'horde.message');
+                }
+            }
         }
     }
 }
@@ -223,11 +292,11 @@ if (!$error && !empty($driver)) {
 if (is_array($next_step)) {
     /* Create a Turba storage instance. */
     $dest = $_SESSION['import_data']['target'];
-    $storage = &Turba_Driver::singleton($dest, $cfgSources[$dest]);
+    $storage = &Turba_Driver::singleton($dest);
     if (is_a($storage, 'PEAR_Error')) {
         $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
     } else {
-        // Purge old address book if requested.
+        /* Purge old address book if requested. */
         if ($_SESSION['import_data']['purge']) {
             $result = $storage->deleteAll();
             if (is_a($result, 'PEAR_Error')) {
@@ -242,14 +311,32 @@ if (is_array($next_step)) {
             if (is_a($row, 'Horde_iCalendar_vcard')) {
                 $row = $storage->toHash($row);
             }
-            $row['__owner'] = Auth::getAuth();
-            $success = $storage->add($row);
 
-            if (is_a($success, 'PEAR_Error')) {
-                $notification->push(sprintf(_("There was an error importing the data: %s"),
-                                            $success->getMessage()), 'horde.error');
+            /* Don't search for empty attributes. */
+            $row = array_filter($row, '_emptyAttributeFilter');
+            $result = $storage->search($row);
+            if (is_a($result, 'PEAR_Error')) {
+                $notification->push($result, 'horde.error');
                 $error = true;
                 break;
+            } elseif ($result->count()) {
+                $result->reset();
+                $object = $result->next();
+                $notification->push(sprintf(_("\"%s\" already exists and was not imported."), $object->getValue('name')), 'horde.message');
+            } else {
+                /* Get share information. */
+                if ($storage->usingShares) {
+                    $row['__owner'] = $storage->share->get('uid');
+                } else {
+                    $row['__owner'] = Auth::getAuth();
+                }
+                $result = $storage->add($row);
+                if (is_a($result, 'PEAR_Error')) {
+                    $notification->push(sprintf(_("There was an error importing the data: %s"),
+                                                $result->getMessage()), 'horde.error');
+                    $error = true;
+                    break;
+                }
             }
         }
         if (!$error) {
@@ -280,10 +367,9 @@ if ($next_step == IMPORT_FILE) {
     /* Build the directory sources select widget. */
     $unique_source = '';
     $source_options = array();
-    foreach ($cfgSources as $key => $entry) {
-        $selected = ($key == $default_source) ? ' selected="selected"' : '';
+    foreach (Turba::getAddressBooks() as $key => $entry) {
         if (!empty($entry['export'])) {
-            $source_options[] = '<option value="' . htmlspecialchars($key) . '"' . $selected . '>' .
+            $source_options[] = '<option value="' . htmlspecialchars($key) . '">' .
                 htmlspecialchars($entry['title']) . "</option>\n";
             $unique_source = $key;
         }
@@ -293,19 +379,28 @@ if ($next_step == IMPORT_FILE) {
     $unique_dest = '';
     $dest_options = array();
     $hasWriteable = false;
-    foreach ($cfgSources as $key => $entry) {
+    foreach (Turba::getAddressBooks(PERMS_EDIT) as $key => $entry) {
         $selected = ($key == $default_source) ? ' selected="selected"' : '';
-        if (empty($entry['readonly']) || (isset($entry['admin']) && in_array(Auth::getAuth(), $entry['admin']))) {
-            $dest_options[] = '<option value="' . htmlspecialchars($key) . '"' . $selected . '>' .
-                htmlspecialchars($entry['title']) . "</option>\n";
-            $unique_dest = $key;
-            $hasWriteable = true;
-        }
+        $dest_options[] = '<option value="' . htmlspecialchars($key) . '" ' . $selected . '>' .
+            htmlspecialchars($entry['title']) . "</option>\n";
+        $unique_dest = $key;
+        $hasWriteable = true;
     }
 
     if (!$hasWriteable) {
         array_shift($templates[$next_step]);
     }
+
+    /* Build the charset options. */
+    $charsets = $nls['encodings'];
+    $all_charsets = $nls['charsets'];
+    natcasesort($all_charsets);
+    foreach ($all_charsets as $charset) {
+        if (!isset($charsets[$charset])) {
+            $charsets[$charset] = $charset;
+        }
+    }
+    $my_charset = NLS::getCharset(true);
 }
 
 foreach ($templates[$next_step] as $template) {
