@@ -1,17 +1,13 @@
 <?php
-
-require_once 'Horde/History.php';
-
 /**
  * The Turba_Driver:: class provides a common abstracted interface to the
  * various directory search drivers.  It includes functions for searching,
  * adding, removing, and modifying directory entries.
  *
- * $Horde: turba/lib/Driver.php,v 1.57.2.12 2005/10/13 21:10:31 selsky Exp $
+ * $Horde: turba/lib/Driver.php,v 1.57.2.22 2006/03/03 16:45:54 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
- * @since   Turba 0.0.1
  * @package Turba
  */
 class Turba_Driver {
@@ -61,26 +57,13 @@ class Turba_Driver {
     var $strict = array();
 
     /**
-     * Whether this source is publicly searchable.
+     * Whether this source stores one address book, or multiple private address
+     * books.
      *
      * @var boolean
      */
     var $public = false;
 
-    /**
-     * Whether this source is read-only (not editable).
-     *
-     * @var boolean
-     */
-    var $readonly = true;
-
-    /**
-     * Any admins for this source.
-     *
-     * @var array
-     */
-    var $admin = array();
-
     /**
      * Hash holding the driver's additional parameters.
      *
@@ -95,6 +78,20 @@ class Turba_Driver {
      */
     var $_capabilities = array();
 
+    /**
+     * Horde_Share object for this source.
+     *
+     * @var Horde_Share
+     */
+     var $share;
+
+     /**
+      * Are Horde_Shares enabled for this source?
+      *
+      * @var boolean
+      */
+     var $usingShares;
+
     /**
      * Constructs a new Turba_Driver object.
      *
@@ -164,22 +161,89 @@ class Turba_Driver {
     {
         $search = array();
         $strict_search = array();
+        $search_terms = array();
+        $subsearch = array();
+        $temp = '';
+        $lastChar = '\"';
+        $glue = '';
+
         foreach ($criteria as $key => $val) {
             if (isset($this->map[$key])) {
                 if (is_array($this->map[$key])) {
-                    $subsearch = array();
+                    /* Composite field, break out the search terms. */
+                    $parts = explode(' ', $val);
+                    if (count($parts) > 1) {
+                        /* Only parse if there was more than 1 search term and
+                         * 'AND' the cumulitive subsearches. */
+                        for ($i = 0; $i < count($parts); $i++) {
+                            $term = $parts[$i];
+                            $firstChar = substr($term, 0, 1);
+                            if ($firstChar =="\"") {
+                                $temp = substr($term, 1, strlen($term) - 1);
+                                $done = false;
+                                while (!$done && $i < count($parts) - 1) {
+                                    $lastChar = substr($parts[$i + 1], -1);
+                                    if ($lastChar =="\"") {
+                                        $temp .= ' ' . substr($parts[$i + 1], 0, -1);
+                                        $done = true;
+                                        $i++;
+                                    } else {
+                                        $temp .= ' ' . $parts[$i + 1];
+                                        $i++;
+                                    }
+                                }
+                                $search_terms[] = $temp;
+                            } else {
+                                $search_terms[] = $term;
+                            }
+                        }
+                        $glue = 'AND';
+                    } else {
+                        /* If only one search term, use original input and
+                           'OR' the searces since we're only looking for 1
+                           term in any of the composite fields. */
+                        $search_terms[0] = $val;
+                        $glue = 'OR';
+                    }
                     foreach ($this->map[$key]['fields'] as $field) {
                         $field = $this->toDriver($field);
                         if (!empty($strict[$field])) {
-                            $strict_search[] = array('field' => $field, 'op' => '=', 'test' => $val);
+                            /* For strict matches, use the original search
+                             * vals. */
+                            $strict_search[] = array('field' => $field,
+                                                     'op' => '=',
+                                                     'test' => $val);
                         } else {
-                            $subsearch[] = array('field' => $field, 'op' => 'LIKE', 'test' => $val);
+                            /* Create a subsearch for each individual search
+                             * term. */
+                            if (count($search_terms) > 1) {
+                                /* Build the 'OR' search for each search term
+                                 * on this field. */
+                                $atomsearch = array();
+                                for ($i = 0; $i < count($search_terms); $i++) {
+                                    $atomsearch[] = array('field' => $field,
+                                                          'op' => 'LIKE',
+                                                          'test' => $search_terms[$i]);
+                                }
+                                $subsearch[] = array('OR' => $atomsearch);
+                                unset($atomsearch);
+                                $glue = 'AND';
+                            } else {
+                                /* $parts may have more than one element, but
+                                 * if they are all quoted we will only have 1
+                                 * $subsearch. */
+                                $subsearch[] = array('field' => $field,
+                                                     'op' => 'LIKE',
+                                                     'test' => $search_terms[0]);
+                                $glue = 'OR';
+                            }
                         }
                     }
                     if (count($subsearch)) {
-                        $search[] = array('OR' => $subsearch);
+                        $search[] = array($glue => $subsearch);
                     }
                 } else {
+                    /* Not a composite field. */
                     if (!empty($strict[$this->map[$key]])) {
                         $strict_search[] = array('field' => $this->map[$key],
                                                  'op' => '=',
@@ -216,7 +280,15 @@ class Turba_Driver {
      */
     function toDriver($attribute)
     {
-        return isset($this->map[$attribute]) && !is_array($this->map[$attribute]) ? $this->map[$attribute] : null;
+        if (!isset($this->map[$attribute])) {
+            return null;
+        }
+
+        if (is_array($this->map[$attribute])) {
+            return $this->map[$attribute]['fields'];
+        } else {
+            return $this->map[$attribute];
+        }
     }
 
     /**
@@ -251,60 +323,97 @@ class Turba_Driver {
     /**
      * Searches the source based on the provided criteria.
      *
-     * TODO: Allow $criteria to contain the comparison operator (<, =, >,
+     * @todo Allow $criteria to contain the comparison operator (<, =, >,
      *       'like') and modify the drivers accordingly.
      *
-     * @param $search_criteria   Hash containing the search criteria.
-     * @param $sort_criteria     The requested sort order which is passed to
-     *                           Turba_List::sort().
-     * @param string $search_type  (optional) Do an AND or an OR search (defaults to AND).
+     * @param array $search_criteria  Hash containing the search criteria.
+     * @param string $sort_criteria   The requested sort order which is passed
+     *                                to Turba_List::sort().
+     * @param string $search_type     Do an AND or an OR search (defaults to
+     *                                AND).
+     * @param array $return_fields    A list of fields to return; defaults to
+     *                                all fields.
+     * @param array $custom_strict    A list of fields that must match exactly.
      *
-     * @return                   The sorted, filtered list of search results.
+     * @return  The sorted, filtered list of search results.
      */
     function &search($search_criteria, $sort_criteria = 'lastname',
-                     $search_type = 'AND', $sort_direction = 0)
+                     $search_type = 'AND', $sort_direction = 0,
+                     $return_fields = array(), $custom_strict = array())
     {
         require_once TURBA_BASE . '/lib/List.php';
         require_once TURBA_BASE . '/lib/Object.php';
 
-        /* If this is not a public source, enforce the requirement
-         * that the source's owner must be equal to the current
-         * user. */
+        /* If we are not using Horde_Share, enfore the requirement that the
+           current user must be the owner of the addressbook. */
         $strict_fields = array();
-        if (!$this->public && !in_array(Auth::getAuth(), $this->admin)) {
-            $search_criteria['__owner'] = Auth::getAuth();
-            $strict_fields = array($this->toDriver('__owner') => true);
+        if ($this->usingShares) {
+            $user = $this->share->get('uid');
+        } else {
+            $user = Auth::getAuth();
         }
+        $search_criteria['__owner'] = $user;
+        $strict_fields = array($this->toDriver('__owner') => true);
 
-        /* Add any fields that much match exactly for this source to
-         * the $strict_fields array. */
+        /* Add any fields that must match exactly for this source to the
+         * $strict_fields array. */
         foreach ($this->strict as $strict_field) {
             $strict_fields[$strict_field] = true;
         }
+        foreach ($custom_strict as $strict_field) {
+            $strict_fields[$this->map[$strict_field]] = true;
+        }
 
-        /* Translate the Turba attributes to a driver-specific search
-         * array. */
+        /* Translate the Turba attributes to driver-specific attributes. */
         $fields = $this->makeSearch($search_criteria, $search_type, $strict_fields);
+        if (count($return_fields)) {
+            $return_fields_pre = array_unique(array_merge(array('__key', '__type', '__owner', 'name'), $return_fields));
+            $return_fields = array();
+            foreach ($return_fields_pre as $field) {
+                $result = $this->toDriver($field);
+                if (is_array($result)) {
+                    foreach ($result as $composite_field) {
+                        $composite_result = $this->toDriver($composite_field);
+                        if ($composite_result) {
+                            $return_fields[] = $composite_result;
+                        }
+                    }
+                } elseif ($result) {
+                    $return_fields[] = $result;
+                }
+            }
+        } else {
+            $return_fields = array_values($this->fields);
+        }
 
         /* Retrieve the search results from the driver. */
-        $results = $this->_search($fields, array_values($this->fields));
+        $objects = $this->_search($fields, $return_fields);
+        if (is_a($objects, 'PEAR_Error')) {
+            return $objects;
+        }
 
         /* Translate the driver-specific fields in the result back to the more
          * generalized common Turba attributes using the map. */
-        $results = $this->toTurbaKeys($results);
-        if (is_a($results, 'PEAR_Error')) {
-            return $results;
-        }
+        $objects = $this->toTurbaKeys($objects);
 
         require_once TURBA_BASE . '/lib/Object.php';
-        require_once TURBA_BASE . '/lib/Group.php';
         $list = &new Turba_List();
-        foreach ($results as $attributes) {
-            if (!empty($attributes['__type']) &&
-                $attributes['__type'] == 'Group') {
-                $list->insert(new Turba_Group($this, $attributes));
-            } else {
-                $list->insert(new Turba_Object($this, $attributes));
+        foreach ($objects as $object) {
+            $done = false;
+            if (!empty($object['__type']) && ucwords($object['__type']) != 'Object') {
+                $type = ucwords($object['__type']);
+                $class = 'Turba_Object_' . $type;
+                if (!class_exists($class)) {
+                    require_once TURBA_BASE . '/lib/Object/' . $type . '.php';
+                }
+
+                if (class_exists($class)) {
+                    $list->insert(new $class($this, $object));
+                    $done = true;
+                }
+            }
+            if (!$done) {
+                $list->insert(new Turba_Object($this, $object));
             }
         }
         $list->sort($sort_criteria, $sort_direction);
@@ -318,7 +427,7 @@ class Turba_Driver {
      *
      * @param array $objectIds  The unique ids of the objects to retrieve.
      *
-     * @return array  The array of retrieved objects (Turba_AbstractObjects).
+     * @return array  The array of retrieved objects (Turba_Objects).
      */
     function &getObjects($objectIds)
     {
@@ -327,18 +436,19 @@ class Turba_Driver {
 
         $objects = $this->_read($criteria, $objectIds, array_values($this->fields));
         if (!is_array($objects)) {
-            return $result = PEAR::raiseError(_("Requested object not found."));
+            $result = PEAR::raiseError(_("Requested object not found."));
+            return $result;
         }
 
         $results = array();
         $objects = $this->toTurbaKeys($objects);
         foreach ($objects as $object) {
             $done = false;
-            if (!empty($object['__type'])) {
+            if (!empty($object['__type']) && ucwords($object['__type']) != 'Object') {
                 $type = ucwords($object['__type']);
-                $class = 'Turba_' . $type;
+                $class = 'Turba_Object_' . $type;
                 if (!class_exists($class)) {
-                    @require_once TURBA_BASE . '/lib/' . $type . '.php';
+                    require_once TURBA_BASE . '/lib/Object/' . $type . '.php';
                 }
 
                 if (class_exists($class)) {
@@ -359,21 +469,23 @@ class Turba_Driver {
      *
      * @param string $objectId  The unique id of the object to retrieve.
      *
-     * @return Turba_AbstractObject  The retrieved object.
+     * @return Turba_Object  The retrieved object.
      */
     function &getObject($objectId)
     {
         $result = &$this->getObjects(array($objectId));
         if (is_a($result, 'PEAR_Error')) {
-            return $result;
+            // Fall through.
         } elseif (empty($result[0])) {
-            return $result = PEAR::raiseError('No results');
+            $result = PEAR::raiseError('No results');
         } else {
+            $result = $result[0];
             if (!isset($this->map['__owner'])) {
-                $result[0]->attributes['__owner'] = Auth::getAuth();
+                $result->attributes['__owner'] = Auth::getAuth();
             }
-            return $result[0];
         }
+
+        return $result;
     }
 
     /**
@@ -386,15 +498,22 @@ class Turba_Driver {
      */
     function add($attributes)
     {
-        if ($this->readonly) {
-            return PEAR::raiseError(_("This address book is read-only"));
-        }
-
+        /* Only set __type and __owner if they are not already set. */
         if (!isset($attributes['__type'])) {
             $attributes['__type'] = 'Object';
         }
-        if (isset($this->map['__owner'])) {
-            $attributes['__owner'] = Auth::getAuth();
+        if (isset($this->map['__owner']) && !isset($attributes['__owner'])) {
+            if (strpos($this->name , ':')) {
+                 list($source, $attributes['__owner']) = explode(':', $this->name, 2);
+            } else {
+                $attributes['__owner'] = Auth::getAuth();
+            }
+        }
+
+        /* Get field values before we translate keys so we can use
+         * them in the history log. */
+        if (isset($attributes['__owner'])) {
+            $turbaOwner = $attributes['__owner'];
         }
         if (!isset($attributes['__uid'])) {
             $attributes['__uid'] = $this->generateUID();
@@ -411,7 +530,7 @@ class Turba_Driver {
 
         /* Log the creation of this item in the history log. */
         $history = &Horde_History::singleton();
-        $history->log('turba:' . (!empty($attributes['__owner']) ? $attributes['__owner'] : Auth::getAuth()) . ':' . $uid, array('action' => 'add'), true);
+        $history->log('turba:' . (!empty($turbaOwner) ? $turbaOwner : Auth::getAuth()) . ':' . $uid, array('action' => 'add'), true);
 
         return $key;
     }
@@ -423,16 +542,12 @@ class Turba_Driver {
      */
     function delete($object_id)
     {
-        if ($this->readonly) {
-            return PEAR::raiseError(_("This address book is read-only"));
-        }
-
         $object = &$this->getObject($object_id);
         if (is_a($object, 'PEAR_Error')) {
             return $object;
         }
 
-        if (!Turba::hasPermission($object, 'object', PERMS_DELETE)) {
+        if (!$object->hasPermission(PERMS_DELETE)) {
             return PEAR::raiseError(_("Permission denied"));
         }
 
@@ -452,32 +567,33 @@ class Turba_Driver {
     }
 
     /**
-     * Deletes all contacts from the address book.
+     * Deletes all contacts from an address book.
      *
-     * @return boolean  True if the operation worked.
+     * @param string  $sourceName  The identifier of the address book to
+     *                             delete.  If omitted, will clear the current
+     *                             user's 'default' address book for this source
+     *                             type.
+     *
+     * @return mixed  True on success, PEAR_Error on failure.
      */
-    function deleteAll()
+    function deleteAll($sourceName = '')
     {
         if (!$this->hasCapability('delete_all')) {
             return PEAR::raiseError('Not supported');
         } else {
-            return $this->_deleteAll();
+            return $this->_deleteAll($sourceName);
         }
     }
 
     /**
      * Modifies an existing entry in the contact source.
      *
-     * @param Turba_AbstractObject $object  The object to update.
+     * @param Turba_Object $object  The object to update.
      *
      * @return string  The object id, possibly updated.
      */
     function save($object)
     {
-        if ($this->readonly) {
-            return PEAR::raiseError(_("Address book is read-only."));
-        }
-
         $attributes = $this->toDriverKeys($object->getAttributes());
         $key = $this->toDriverKeys(array('__key' => $object->getValue('__key')));
         list($object_key, $object_id) = each($key);
@@ -497,6 +613,26 @@ class Turba_Driver {
         return $object_id;
     }
 
+    /**
+     * Returns the number of contacts of the current user in this address book.
+     *
+     * @return integer  The number of contacts that the user owns.
+     */
+    function countContacts()
+    {
+        static $count;
+
+        if (!isset($count)) {
+            $count = $this->_search(array('AND' => array(array('field' => $this->toDriver('__owner'), 'op' => '=', 'test' => Auth::getAuth()))), array($this->toDriver('__key')));
+            if (is_a($count, 'PEAR_Error')) {
+                return $count;
+            }
+            $count = count($count);
+        }
+
+        return $count;
+    }
+
     /**
      * Returns the criteria available for this source except '__key'.
      *
@@ -535,9 +671,9 @@ class Turba_Driver {
     }
 
     /**
-     * Exports a given Turba_AbstractObject as an iCalendar vCard.
+     * Exports a given Turba_Object as an iCalendar vCard.
      *
-     * @param Turba_AbstractObject $object  A Turba_AbstractObject.
+     * @param Turba_Object $object  A Turba_Object.
      *
      * @return Horde_iCalendar_vcard  A Horde_iCalendar_vcard object.
      */
@@ -648,7 +784,7 @@ class Turba_Driver {
      *
      * @return array  A Turba attribute hash.
      */
-    function toHash($vcard)
+    function toHash(&$vcard)
     {
         if (!is_a($vcard, 'Horde_iCalendar_vcard')) {
             return PEAR::raiseError('Invalid parameter for Turba_Driver::toHash(), expected Horde_iCalendar_vcard object.');
@@ -810,6 +946,36 @@ class Turba_Driver {
         return $hash;
     }
 
+    /**
+     * Checks if the current user has the requested permissions on this
+     * source.
+     *
+     * @param integer $perm  The permission to check for.
+     *
+     * @return boolean  True if the user has permission, otherwise false.
+     */
+     function hasPermission($perm)
+     {
+         if ($this->usingShares) {
+             return $this->share->hasPermission(Auth::getAuth(), $perm);
+         } else {
+             return Turba::hasPermission($this->name, 'source', $perm);
+         }
+     }
+
+    /**
+     * Creates an object key for a new object.
+     *
+     * @param array $attributes  The attributes (in driver keys) of the
+     *                           object being added.
+     *
+     * @return string  A unique ID for the new object.
+     */
+    function _makeKey($attributes)
+    {
+        return md5(mt_rand());
+    }
+
     /**
      * Static method to construct Turba_Driver objects. Use this so that we
      * can return PEAR_Error objects if anything goes wrong.
@@ -832,7 +998,8 @@ class Turba_Driver {
         if (class_exists($class)) {
             $driver = &new $class($config['params']);
         } else {
-            return $driver = PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class));
+            $driver = PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class));
+            return $driver;
         }
 
         $result = $driver->_init();
@@ -862,18 +1029,26 @@ class Turba_Driver {
             $driver->strict = $config['strict'];
         }
 
-        /* Store admins. */
-        if (isset($config['admin'])) {
-            $driver->admin = $config['admin'];
-        }
-
-        /* Set flags. */
-        if (isset($config['public'])) {
-            $driver->public = $config['public'];
-        }
-        if (isset($config['readonly'])) {
-            $driver->readonly = $config['readonly'] &&
-                (!isset($config['admin']) || !in_array(Auth::getAuth(), $config['admin']));
+        /* Get a share object if we need it. */
+        if (!empty($config['use_shares'])) {
+            if ((strpos($name, ':') === false)) {
+                if (!$GLOBALS['turba_shares']->exists($name . ':' . Auth::getAuth())) {
+                    /* User's 'default' share for this source type not present */
+                    $params = array('sourceType' => $name);
+                    $share = &Turba::createShare($params, true);
+                    if (is_a($share, 'PEAR_Error')) {
+                        return $share;
+                    }
+                }
+                $name .= ':' . Auth::getAuth();
+            }
+            $driver->share = &$GLOBALS['turba_shares']->getShare($name);
+            if (is_a($driver->share, 'PEAR_Error')) {
+                return $driver->share;
+            }
+            $driver->usingShares = true;
+        } else {
+            $driver->usingShares = false;
         }
 
         return $driver;
@@ -889,18 +1064,20 @@ class Turba_Driver {
      *
      * @param string $name   String containing the internal name of this
      *                       source.
-     * @param array $config  Array containing the configuration information for
-     *                       this source.
      *
      * @return Turba_Driver  The concrete Turba_Driver reference, or a
      *                       PEAR_Error on error.
      */
-    function &singleton($name, $config)
+    function &singleton($name)
     {
         static $instances = array();
 
         if (!isset($instances[$name])) {
-            $instances[$name] = &Turba_Driver::factory($name, $config);
+            if (!isset($GLOBALS['cfgSources'][$name])) {
+                $error = PEAR::raiseError(sprintf(_("The address book \"%s\" does not exist."), $name));
+                return $error;
+            }
+            $instances[$name] = &Turba_Driver::factory($name, $GLOBALS['cfgSources'][$name]);
         }
 
         return $instances[$name];
