@@ -5,7 +5,7 @@ require_once 'Horde/Cache.php';
 /**
  * Turba directory driver implementation for an IMSP server.
  *
- * $Horde: turba/lib/Driver/imsp.php,v 1.21.4.8 2005/06/22 18:03:40 mrubinsk Exp $
+ * $Horde: turba/lib/Driver/imsp.php,v 1.21.4.11 2005/11/10 12:44:30 mdjukic Exp $
  *
  * @author  Michael Rubinsky <mrubinsk@horde.org>
  * @package Turba
@@ -66,6 +66,8 @@ class Turba_Driver_imsp extends Turba_Driver {
         $this->_groupField  = $params['group_id_field'];
         $this->_groupValue  = $params['group_id_value'];
         $this->_bookName    = $params['name'];
+        $this->_myRights    = $params['my_rights'];
+        $this->_perms       = $this->_aclToHordePerms($params['my_rights']);
     }
 
     /**
@@ -162,10 +164,13 @@ class Turba_Driver_imsp extends Turba_Driver {
         $id = array_values($id);
         $idCount = count($id);
         $result = array();
+        $members = array();
+        $tmembers = array();
+
         for ($i = 0; $i < $idCount; $i++) {
             $temp = $this->_imsp->getEntry($this->_bookName, $id[$i]);
             if (is_a($temp, 'PEAR_Error')) {
-                $result = array();
+                return $temp;
             } else {
                 $temp['fullname'] = $temp['name'];
                 $isIMSPGroup = false;
@@ -177,7 +182,8 @@ class Turba_Driver_imsp extends Turba_Driver {
                     ($temp[$this->_groupField] == $this->_groupValue)) {
                     $isIMSPGroup = true;
                 }
-
+                // Get the group members that might have been added from other
+                // IMSP applications.
                 if ($isIMSPGroup) {
                     if (isset($temp['email'])) {
                         $emailList = $this->_getGroupEmails($temp['email']);
@@ -190,10 +196,12 @@ class Turba_Driver_imsp extends Turba_Driver {
                                 $members[] = $memberName[0];
                             }
                         }
-
-                        $temp['__members'] = serialize($members);
                     }
-
+                    if (isset($temp['__members'])) {
+                        $tmembers = unserialize($temp['__members']);
+                    }
+                    $temp['__members'] = serialize($this->_removeDuplicated(
+                                                   array($members, $tmembers)));
                     $temp['__type'] = 'Group';
                     $temp['email'] = null;
                     $result = $temp;
@@ -219,7 +227,7 @@ class Turba_Driver_imsp extends Turba_Driver {
      */
     function _add($attributes)
     {
-        /* We need to map out Turba_Groups back to IMSP groups before
+        /* We need to map out Turba_Object_Groups back to IMSP groups before
          * writing out to the server. We need to array_values() it in
          * case an entry was deleted from the group. */
         if ($attributes['__type'] == 'Group') {
@@ -241,14 +249,15 @@ class Turba_Driver_imsp extends Turba_Driver {
                 $result = $this->_read('name', $members, array('email'));
                 $count = count($result);
                 for ($i = 0; $i < $count; $i++) {
-                    $contact = sprintf("%s<%s>\n", $members[$i],
-                                       $result[$i]['email']);
-                    $attributes['email'] .= $contact;
+                    if (isset($result[$i]['email'])) {
+                        $contact = sprintf("%s<%s>\n", $members[$i],
+                                           $result[$i]['email']);
+                        $attributes['email'] .= $contact;
+                    }
                 }
             }
         }
 
-        unset($attributes['__members']);
         unset($attributes['__type']);
         unset($attributes['fullname']);
         if (!$this->params['contact_ownership']) {
@@ -404,7 +413,7 @@ class Turba_Driver_imsp extends Turba_Driver {
             break;
         }
 
-        /* Are we searching for only Turba_Groups or Turba_Objects?
+        /* Are we searching for only Turba_Object_Groups or Turba_Objects?
          * This is needed so the 'Show Lists' and 'Show Contacts'
          * links work correctly in Turba. */
         if ($searchkey == '__type') {
@@ -444,7 +453,7 @@ class Turba_Driver_imsp extends Turba_Driver {
 
         /* Finally get to the command.  Check the cache first, since each 'Turba'
            search may consist of a number of identical IMSP searchaddress calls in
-           order for the AND and OR parts to work correctly.  15 Second lifetime 
+           order for the AND and OR parts to work correctly.  15 Second lifetime
            should be reasonable for this. This should reduce load on IMSP server
            somewhat.*/
         $results = $cache->get($key, 15);
@@ -510,4 +519,42 @@ class Turba_Driver_imsp extends Turba_Driver {
         }
         return array_unique($unames);
     }
+
+    /**
+     * Checks if the current user has the requested permission
+     * on this source.
+     *
+     * @param integer $perm  The permission to check for.
+     *
+     * @return boolean  true if user has permission, false otherwise.
+     */
+     function hasPermission($perm)
+     {
+         return $this->_perms & $perm;
+     }
+
+     /**
+      * Converts an acl string to a Horde Permissions bitmask.
+      *
+      * @param string $acl  A standard, IMAP style acl string.
+      *
+      * @return integer  Horde Permissions bitmask.
+      */
+     function _aclToHordePerms($acl)
+     {
+         $hPerms = 0;
+         if (strpos($acl, 'w') !== false) {
+             $hPerms |= PERMS_EDIT;
+         }
+         if (strpos($acl, 'r') !== false) {
+             $hPerms |= PERMS_READ;
+         }
+         if (strpos($acl, 'd') !== false) {
+             $hPerms |= PERMS_DELETE;
+         }
+         if (strpos($acl, 'l') !== false) {
+             $hPerms |= PERMS_SHOW;
+         }
+         return $hPerms;
+     }
 }
